unit uFRename;
{$INCLUDE 'FRename3.inc'}

interface

uses
  Windows, SysUtils, Classes,
  {$IFDEF FAR3} Plugin3, {$ELSE} PluginW, {$ENDIF} FarKeysW, FarColor,
  uLog, uFAR;

procedure SetStartupInfoW(const Info: TPluginStartupInfo); stdcall;
procedure ExitFARW( {$IFDEF FAR3_UP} const Info: TExitInfo {$ENDIF} ); stdcall;
{$IFDEF FAR3_UP}
procedure GetGlobalInfoW(var Info: TGlobalInfo); stdcall;
function OpenW(var Info: TOpenInfo): THandle; stdcall;
{$ELSE}
function OpenPluginW(OpenFrom: integer; Item: integer): THandle; stdcall;
{$ENDIF}
procedure GetPluginInfoW(var Info: TPluginInfo); stdcall;
function ProcessEditorEventW( {$IFDEF FAR3_UP} const Info: TProcessEditorEventInfo {$ELSE} Event: integer; Param: Pointer {$ENDIF} ): TIntPtr; stdcall;
function ProcessEditorInputW( {$IFDEF FAR3_UP} const Info: TProcessEditorInputInfo {$ELSE} const Info: TInputRecord {$ENDIF} ): TIntPtr; stdcall;

implementation

uses
  uSystem, uMessages, uFRenameConsts, uFileList;

const
  PLUGIN_TITLE = 'FRename3';
  PLUGIN_DESCRIPTION = 'Files Rename 3';
  PLUGIN_AUTHOR = 'Pepak, http://www.pepak.net';

function FindEditor(EditorID: integer; out List: TFileList): boolean;
var
  Info: TEditorInfo;
  FileName: string;
begin
  Result := False;
  if EditorID = -1 then
    if TFarEditor.GetInfo(EditorID, Info) then
      EditorID := Info.EditorID;
  if TFileList.FindActiveFileListByEditorID(Info.EditorID, List) then
    Result := True
  else
    if TFarEditor.GetFileName(EditorID, FileName) then
      if TFileList.FindActiveFileListByListFileName(FileName, List) then
        Result := True;
end;

procedure SetStartupInfoW(const Info: TPluginStartupInfo);
begin
  FarApi := Info;
  FSF := Info.FSF^;
  FarApi.FSF := @FSF;
end;

procedure ExitFARW( {$IFDEF FAR3_UP} const Info: TExitInfo {$ENDIF} );
begin
end;

{$IFDEF FAR3_UP}
procedure GetGlobalInfoW(var Info: TGlobalInfo);
var
  FN: string;
  VerHigh, VerLow, VerBuild: LongWord;
begin
  FillChar(Info, Sizeof(Info), 0);
  Info.StructSize := Sizeof(Info);
  Info.MinFarVersion := MakeFarVersion( {$IFDEF FAR3_UP} 3, 0, 0, 0, 0 {$ELSE} 2, 0, 1807 {$ENDIF} );
  FN := GetModuleName(HInstance);
  if GetFileVersion(FN, VerHigh, VerLow, VerBuild) then
    Info.Version := MakeFarVersion( {$IFDEF FAR3_UP} VerHigh, VerLow, 0, VerBuild, 0 {$ELSE} VerHigh, VerLow, VerBuild {$ENDIF} )
  else
    Info.Version := MakeFarVersion( {$IFDEF FAR3_UP} 0, 0, 0, 0, 0 {$ELSE} 0, 0 ,0 {$ENDIF} );
  Info.Guid := PLUGIN_GUID;
  Info.Title := PLUGIN_TITLE;
  Info.Description := PLUGIN_DESCRIPTION;
  Info.Author := PLUGIN_AUTHOR;
end;
{$ENDIF}

var
  PluginInfoInitialized: boolean = False;
  PanelMenuStrings:  array[0..0] of PFarChar;
  PanelMenuGUIDs:    array[0..0] of TGUID;
  //ConfigMenuStrings: array[0..0] of PFarChar;
  //ConfigMenuGUIDs:   array[0..0] of TGUID;

procedure GetPluginInfoW(var Info: TPluginInfo);
begin
  // Initialize messages and GUIDs
  if not PluginInfoInitialized then begin
    PanelMenuStrings[0] := GetMsg(MTitle);
    PanelMenuGUIDs[0] := MENU_PANEL_GUID;
    //ConfigMenuStrings[0] := GetMsg(MTitle);
    //ConfigMenuGUIDs[0] := MENU_CONFIG_GUID;
    PluginInfoInitialized := True;
  end;
  FillChar(Info, Sizeof(Info), 0);
  Info.StructSize := Sizeof(Info);
  Info.Flags := PF_EDITOR;
  {$IFDEF FAR3_UP}
  Info.PluginMenu.Count := Length(PanelMenuStrings);
  Info.PluginMenu.Strings := @PanelMenuStrings[0];
  Info.PluginMenu.Guids := @PanelMenuGUIDs[0];
  //Info.PluginConfig.Count := Length(ConfigMenuStrings);
  //Info.PluginConfig.Strings := @ConfigMenuStrings[0];
  //Info.PluginConfig.Guids := @ConfigMenuGUIDs[0];
  {$ELSE}
  Info.PluginMenuStringsNumber := Length(PanelMenuStrings);
  Info.PluginMenuStrings := @PanelMenuStrings[0];
  //Info.PluginConfigStringsNumber := Length(ConfigMenuStrings);
  //Info.PluginConfigStrings := @ConfigMenuStrings[0];
  {$ENDIF}
end;

{$IFDEF FAR3_UP}
function OpenW(var Info: TOpenInfo): THandle;
{$ELSE}
function OpenPluginW(OpenFrom: integer; Item: integer): THandle;
{$ENDIF}
var
  List: TFileList;
  FileName: string;
begin
  Result := 0;
  if {$IFDEF FAR3_UP} Info.OpenFrom = OPEN_EDITOR {$ELSE} OpenFrom = OPEN_EDITOR {$ENDIF} then begin
    if TFileList.FindActiveFileListByEditorID(-1, List) then
      List.ShowEditorMenu
    else if TFarEditor.GetFileName(-1, FileName) and TFileList.FindActiveFileListByListFileName(FileName, List) then
      List.ShowEditorMenu;
  end
  else begin
    List := TFileList.Create;
    try
      if not List.LoadFromPanel(True) then
        FarMessage([MTitle, MFailedToLoadFileList, MOK], FMSG_WARNING, 1)
      else if not List.OpenInEditor then
        // Nothing to do
      else
        List := nil; // This prevents the editor from being destroyed
    finally
      FreeAndNil(List);
    end;
  end;
end;

function ProcessEditorEventW( {$IFDEF FAR3_UP} const Info: TProcessEditorEventInfo {$ELSE} Event: integer; Param: Pointer {$ENDIF} ): TIntPtr;
var
  List: TFileList;
begin
  Result := 0;
  {$IFDEF FAR3_UP}
  if Info.StructSize >= Sizeof(Info) then
  {$ENDIF}
    if {$IFDEF FAR3_UP} Info.Event {$ELSE} Event {$ENDIF} = EE_CLOSE then begin
      if FindEditor( {$IFDEF FAR3_UP} Info.EditorID {$ELSE} -1 {$ENDIF} , List) then
        try
          List.RenameFiles;
        finally
          FreeAndNil(List);
        end;
    end;
end;

function ProcessEditorInputW( {$IFDEF FAR3_UP} const Info: TProcessEditorInputInfo {$ELSE} const Info: TInputRecord {$ENDIF} ): TIntPtr;
var
  ShiftFlags: DWORD;
  P: PInputRecord;
  List: TFileList;
begin
  Result := 0;
  {$IFDEF FAR3_UP}
  P := @Info.Rec;
  {$ELSE}
  P := @Info;
  {$ENDIF}
  {$IFDEF FAR3_UP}
  if Info.StructSize >= Sizeof(Info) then
  {$ENDIF}
    if P^.EventType = KEY_EVENT then
      if P^.Event.KeyEvent.bKeyDown then begin
        ShiftFlags := P^.Event.KeyEvent.dwControlKeyState and (LEFT_ALT_PRESSED or LEFT_CTRL_PRESSED or RIGHT_ALT_PRESSED or RIGHT_CTRL_PRESSED or SHIFT_PRESSED);
        if (ShiftFlags = LEFT_ALT_PRESSED) or (ShiftFlags = RIGHT_ALT_PRESSED) then begin
          if P^.Event.KeyEvent.wVirtualKeyCode = VK_F1 then begin
            if FindEditor(-1, List) then
              if List.JumpToDestinationFileName then
                Result := 1;
          end
          else if P^.Event.KeyEvent.wVirtualKeyCode = VK_F2 then begin
            if FindEditor(-1, List) then
              if List.SelectDestinationFilenames then
                Result := 1;
          end
        end;
      end;
end;

exports
  SetStartupInfoW,
  ExitFARW,
  {$IFDEF FAR3_UP}
  GetGlobalInfoW,
  OpenW,
  {$ELSE}
  OpenPluginW,
  {$ENDIF}
  GetPluginInfoW,
  ProcessEditorEventW,
  ProcessEditorInputW;

initialization
  {$IFDEF FAR3_UP}
  uFar.PluginID := PLUGIN_GUID;
  {$ENDIF}

end.
